<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width" />
    <title>Login</title>
    <style>
      body {
        background-color: black;
        background-size: cover;
        background-repeat: no-repeat;
      }
      #root {
        margin-top: 3em;
      }
    </style>
  </head>
  <body>
    <div id="root">
    </div>

    <script src="https://cdn.auth0.com/js/lock/11.3.1/lock.min.js"></script>
    <script>
      var lock = new Auth0Lock('{{ auth0_config["client_id"] }}', '{{ auth0_config["client_url"] }}', {
        container: 'root',
        auth: {
          redirectUrl: '{{ auth0_config["callback_uri"] }}',
          responseType: 'code',
          params: {
            scope: 'openid email' // Learn about scopes: https://auth0.com/docs/scopes
          }
        },
        allowSignUp: true,
        languageDictionary: {
          emailInputPlaceholder: "name@company.com",
          title: ""
        },
      });
      lock.show();
    </script>
  </body>
</html>
